import wx
from gooey.gui.pubsub import pub
from gooey.gui import events

from gooey.gui.util import wx_util


class Sidebar(wx.Panel):

  def __init__(self, parent, *args, **kwargs):
    super(Sidebar, self).__init__(parent, *args, **kwargs)
    self.SetDoubleBuffered(True)

    self._parent = parent

    self._do_layout()

  def _do_layout(self):
    self.SetDoubleBuffered(True)
    self.SetBackgroundColour('#f2f2f2')
    self.SetSize((180, 0))
    self.SetMinSize((180, 0))

    STD_LAYOUT = (0, wx.LEFT | wx.RIGHT | wx.EXPAND, 10)

    container = wx.BoxSizer(wx.VERTICAL)
    container.AddSpacer(15)
    container.Add(wx_util.h1(self, 'Actions'), *STD_LAYOUT)
    container.AddSpacer(5)
    self.listbox = wx.ListBox(self, -1)
    container.Add(self.listbox, 1, wx.LEFT | wx.RIGHT | wx.EXPAND, 10)
    container.AddSpacer(20)
    self.SetSizer(container)

    self.Bind(wx.EVT_LISTBOX, self.selection_change, self.listbox)

  def set_list_contents(self, contents):
    self.listbox.AppendItems(contents)
    self.listbox.SetSelection(0)

  def selection_change(self, evt):
    pub.send_message(
      events.LIST_BOX,
      selection=self.listbox.GetItems()[self.listbox.GetSelection()])
    evt.Skip()



import wx

class FileDrop(wx.FileDropTarget):
  def __init__(self, window):
    wx.FileDropTarget.__init__(self)
    self.window = window

  def OnDropFiles(self, x, y, filenames):
    for name in filenames:
      self.window.WriteText(name)



import sys

from gooey.gui.processor import ProcessController
from gooey.gui.model import States
from gooey.gui.pubsub import pub
from gooey.gui import events
from gooey.gui.windows import layouts

import wx

class Presenter(object):
  def __init__(self, view, model):
    self.view = view
    self.model = model

    self.client_runner = ProcessController(
      self.model.progress_regex,
      self.model.progress_expr
    )

    pub.subscribe(self.on_cancel, events.WINDOW_CANCEL)
    pub.subscribe(self.on_stop, events.WINDOW_STOP)
    pub.subscribe(self.on_start, events.WINDOW_START)
    pub.subscribe(self.on_restart,  events.WINDOW_RESTART)
    pub.subscribe(self.on_edit, events.WINDOW_EDIT)
    pub.subscribe(self.on_close, events.WINDOW_CLOSE)

    # console statuses from the other thread
    pub.subscribe(self.on_new_message, 'console_update')
    pub.subscribe(self.on_progress_change, 'progress_update')
    pub.subscribe(self.on_client_done, 'execution_complete')


    pub.subscribe(self.on_selection_change, events.LIST_BOX)


  def on_selection_change(self, selection):
    self.update_model()
    self.model.active_group = selection
    self.redraw_from_model()
    self.syncronize_from_model()

  def initialize_view(self):
    self.view.window_title = self.model.program_name
    self.view.window_size = self.model.default_size

    self.view.required_section.clear()
    self.view.optional_section.clear()
    self.view.required_section.populate(self.model.required_args, self.model.num_required_cols)
    self.view.optional_section.populate(self.model.optional_args, self.model.num_optional_cols)

    if self.model.use_monospace_font:
      self.view.set_display_font_style('monospace')

    if self.should_disable_stop_button():
      self.view.disable_stop_button()
    else:
      self.view.enable_stop_button()

    if self.model.layout_type == layouts.COLUMN:
      self.view.set_list_contents(list(self.model.argument_groups.keys()))

    if self.model.auto_start:
      self.model.update_state(States.RUNNNING)
      self.on_start()
    self.syncronize_from_model()

  def update_model(self):
    self.update_list(self.model.required_args, self.view.required_section.get_values())
    self.update_list(self.model.optional_args, self.view.optional_section.get_values())
    self.syncronize_from_model()

  def syncronize_from_model(self):
    #TODO move this out of the presenter
    #TODO Make all view interactions thread safe
    wx.CallAfter(self.syncronize_from_model_async)

  def syncronize_from_model_async(self):
    # update heading titles
    self.view.heading_title = self.model.heading_title
    self.view.heading_subtitle = self.model.heading_subtitle

    # refresh the widgets
    for index, widget in enumerate(self.view.required_section):
      widget.set_value(self.model.required_args[index]._value)
    for index, widget in enumerate(self.view.optional_section):
      widget.set_value(self.model.optional_args[index]._value)

    # swap the views
    getattr(self, self.model.current_state)()

  def redraw_from_model(self):
    self.view.freeze()
    self.view.required_section.clear()
    self.view.optional_section.clear()
    self.view.required_section.populate(self.model.required_args, self.model.num_required_cols)
    self.view.optional_section.populate(self.model.optional_args, self.model.num_optional_cols)
    getattr(self, self.model.current_state)()
    self.view.thaw()

  def should_disable_stop_button(self):
    return self.model.stop_button_disabled

  def on_start(self):
    self.update_model()
    if not self.model.is_valid():
      return self.view.show_missing_args_dialog()
    command = self.model.build_command_line_string()
    self.client_runner.run(command)
    self.model.update_state(States.RUNNNING)
    self.syncronize_from_model()

  def on_stop(self):
    self.ask_stop()

  def on_edit(self):
    self.model.update_state(States.CONFIGURING)
    self.syncronize_from_model()

  def on_restart(self):
    self.on_start()

  def on_cancel(self):
    if self.view.confirm_exit_dialog():
      self.view.Destroy()
      sys.exit()

  def on_close(self):
    self.view.Destroy()
    sys.exit()

  def on_new_message(self, msg):
    # observes changes coming from the subprocess
    self.view.update_console_async(msg)

  def on_progress_change(self, progress):
    # observes changes coming from the subprocess
    self.view.update_progress_aync(progress, self.model.disable_progress_bar_animation)

  def on_client_done(self):
    if self.client_runner.was_success():
      self.model.update_state(States.SUCCESS)
    else:
      self.model.update_state(States.ERROR)
    self.syncronize_from_model()

  def ask_stop(self):
    if self.view.confirm_stop_dialog():
      self.stop()
      return True
    return False

  def stop(self):
    self.client_runner.stop()

  def configuring(self):
    self.view.hide_all_buttons()
    self.view.hide('check_mark', 'running_img', 'error_symbol', 'runtime_display')
    self.view.show('settings_img', 'cancel_button', 'start_button', 'config_panel')
    self.view.Layout()

  def running(self):
    self.view.hide_all_buttons()
    self.view.hide('check_mark', 'settings_img', 'error_symbol', 'config_panel')
    self.view.show('running_img', 'stop_button', 'progress_bar', 'runtime_display')
    self.view.progress_bar.Pulse()
    self.view.Layout()

  def success(self):
    self.view.hide_all_buttons()
    self.view.hide('running_img', 'progress_bar', 'config_panel')
    self.view.show('check_mark', 'edit_button', 'restart_button', 'close_button', 'runtime_display')
    self.view.Layout()

  def error(self):
    self.view.hide_all_buttons()
    self.view.hide('running_img', 'progress_bar', 'config_panel')
    self.view.show('error_symbol', 'edit_button', 'restart_button', 'close_button', 'runtime_display')
    self.view.Layout()

  @staticmethod
  def partition(collection, condition):
    return filter(condition, collection), filter(lambda x: not condition(x), collection)

  def update_list(self, collection, new_values):
    # convenience method for syncronizing the model -> widget list collections
    for index, val in enumerate(new_values):
      collection[index].value = val






"""
Naval Fate.

Usage:
  naval_fate.py ship new <name>...
  naval_fate.py ship <name> move <x> <y> [--speed=<kn>]
  naval_fate.py ship shoot <x> <y>
  naval_fate.py mine (set|remove) <x> <y> [--moored|--drifting]
  naval_fate.py -h | --help
  naval_fate.py --version

Options:
  -h --help     Show this screen.
  --version     Show version.
  --speed=<kn>  Speed in knots [default: 10].
  --moored      Moored (anchored) mine.
  --drifting    Drifting mine.

"""

# Standard
# choice
# counter
# flag
# mutually_exclusive
#
# types?


import re
from docopt import docopt, Option, Argument


class MyOption(Option):
  def __init__(self, *args, **kwargs):
    self.description = kwargs.pop('description', None)
    super(MyOption, self).__init__(*args, **kwargs)


  @classmethod
  def parse(class_, option_description):
      short, long, argcount, value = None, None, 0, False
      options, _, description = option_description.strip().partition('  ')
      options = options.replace(',', ' ').replace('=', ' ')
      for s in options.split():
          if s.startswith('--'):
              long = s
          elif s.startswith('-'):
              short = s
          else:
              argcount = 1
      if argcount:
          matched = re.findall(r'\[default: (.*)\]', description, flags=re.I)
          value = matched[0] if matched else None
      return class_(short, long, argcount, value, description=description.strip())

  def __repr__(self):
      return 'Option(%r, %r, %r, %r, %r)' % (self.short, self.long,
                                       self.argcount, self.value, self.description)


if __name__ == '__main__':
    import sys
    a = docopt(__doc__)
    # import re
    # doc = __doc__
    # split = re.split('\n *(<\S+?>|-\S+?)', doc)[1:]
    # split = [s1 + s2 for s1, s2 in zip(split[::2], split[1::2])]
    # options = [MyOption.parse(s) for s in split if s.startswith('-')]
    # arguments = [Argument.parse(s) for s in split if s.startswith('<')]
    # #return options, arguments
    # print arguments
    # print options
    print a
    a = 10



"""
App wide event registry

Everything in the application is communitcated via pubsub. These are the events that
tie everythign together.
"""
import wx

new_id = lambda: str(wx.NewId())

WINDOW_STOP     = new_id()
WINDOW_CANCEL   = new_id()
WINDOW_CLOSE    = new_id()
WINDOW_START    = new_id()
WINDOW_RESTART  = new_id()
WINDOW_EDIT     = new_id()

WINDOW_CHANGE   = new_id()
PANEL_CHANGE    = new_id()
LIST_BOX        = new_id()



'''
Created on Jan 25, 2014

@author: Chris

Provides Internationalization for all text within the program.

'''

import io
import os
import json

__all__ = ['load', '_']

_DICTIONARY = None

def load(language_dir, filename):
  ''' Open and return the supplied json file '''
  global _DICTIONARY
  try:
    json_file = filename + '.json'
    with io.open(os.path.join(language_dir, json_file), 'r', encoding='utf-8') as f:
      _DICTIONARY = json.load(f)
  except IOError:
    raise IOError('{0} Language file not found at location {1}. '
                  'Make sure that your translation file is in the '
                  'listed language directory'.format(filename.title(), language_dir))

def translate(key):
  return _DICTIONARY.get(key, key)

def _(key):
  return translate(key)



'''
Collection of the image paths.

The module is meant to act as a singleton, hence the globals() abuse.

Image credit: kidcomic.net
'''
from functools import partial
import os
from gooey.gui.util.freeze import get_resource_path

_image_details = (
  ('program_icon', 'program_icon.ico'),
  ('success_icon', 'success_icon.png'),
  ('running_icon', 'running_icon.png'),
  ('loading_icon', 'loading_icon.gif'),
  ('config_icon', 'config_icon.png'),
  ('error_icon', 'error_icon.png')
)

def init(image_dir):
  ''' initalize the images from the default directory path '''
  defaults = {variable_name: os.path.join(image_dir, filename)
              for variable_name, filename in _image_details}
  globals().update(defaults)

def patch_images(new_image_dir):
  '''
  Loads custom images from the user supplied directory
  '''
  pathto = partial(os.path.join, new_image_dir)

  if new_image_dir != 'default':
    if not os.path.isdir(new_image_dir):
      raise IOError('Unable to find the user supplied directory {}'.format(new_image_dir))

    new_images = ((varname, pathto(filename))
                           for varname, filename in _image_details
                           if os.path.exists(pathto(filename)))
    # push the changes into module scope
    globals().update(new_images)


default_dir = get_resource_path('images')
init(default_dir)

















'''
Image Repository acts as a funky dynamic singlton module.
'''
import os
import pytest
import tempfile


def test_variable_names_are_pushed_to_module_scope(expected_attrs):
  '''
  The dynamically initialized Globals() should contain the expected images at runtime
  '''
  from gooey.gui import image_repository
  assert all((attr in image_repository.__dict__) for attr in expected_attrs)


def test_patch_returns_error_on_invalid_dir():
  '''
  patch should explode with a helpful message if it
  cannot find the supplied directory
  '''
  from gooey.gui import image_repository

  with pytest.raises(IOError) as kaboom:
    image_repository.patch_images('foo/bar/not/a/path')

  # our error
  assert ' user supplied' in str(kaboom.value)
  assert 'foo/bar/not/a/path' in str(kaboom.value)


def test_module_scope_is_updated_on_patch(expected_attrs):
  '''
  Patch should update the module's globals() on success
  '''
  from gooey.gui import image_repository
  testing_icons = ('config_icon.png', 'success_icon.png')
  try:
    # setup
    make_user_files(*testing_icons)
    old_icon = image_repository.config_icon
    # load up our new icon(s)
    image_repository.patch_images(tempfile.tempdir)
    new_icon = image_repository.config_icon
    assert old_icon != new_icon
  finally:
    cleanup_temp(*testing_icons)


# helpers
def make_user_files(*filenames):
  for filename in filenames:
    with open(os.path.join(tempfile.gettempdir(), filename), 'w') as f:
      f.write('temp')

def cleanup_temp(*filenames):
  for filename in filenames:
    os.remove(os.path.join(tempfile.gettempdir(), filename))



'''
Main runner entry point for Gooey.
'''

import wx

from gooey.gui.lang import i18n
from gooey.gui.controller import Controller

from gooey.gui import image_repository



def run(build_spec):
  app = wx.App(False)

  i18n.load(build_spec['language_dir'], build_spec['language'])
  image_repository.patch_images(build_spec['image_dir'])
  controller = Controller(build_spec)
  controller.run()
  app.MainLoop()





import re

from gooey.gui.processor import ProcessController


def test_extract_progress():
  # should pull out a number based on the supplied
  # regex and expression
  processor = ProcessController(r"^progress: (\d+)%$", None)
  assert processor._extract_progress('progress: 50%') == 50

  processor = ProcessController(r"total: (\d+)%$", None)
  assert processor._extract_progress('my cool total: 100%') == 100

def test_extract_progress_returns_none_if_no_regex_supplied():
  processor = ProcessController(None, None)
  assert processor._extract_progress('Total progress: 100%') == None

def test_extract_progress_returns_none_if_no_match_found():
  processor = ProcessController(r'(\d+)%$', None)
  assert processor._extract_progress('No match in dis string') == None

def test_eval_progress():
  # given a match in the string, should eval the result
  regex = r'(\d+)/(\d+)$'
  processor = ProcessController(regex, r'x[0] / x[1]')
  match = re.search(regex, '50/50')
  assert processor._eval_progress(match) == 1.0

def test_eval_progress_returns_none_on_failure():
  # given a match in the string, should eval the result
  regex = r'(\d+)/(\d+)$'
  processor = ProcessController(regex, r'x[0] *^/* x[1]')
  match = re.search(regex, '50/50')
  assert processor._eval_progress(match) == None



'''
Util for supporting WxPython 3 & 4
'''

import wx
try:
    import wx.adv
except ImportError:
    pass

isLatestVersion = wx.version().startswith('4')


class Constants:
    if isLatestVersion:
        WX_FONTSTYLE_NORMAL = wx.FONTSTYLE_NORMAL
        WX_DP_DROPDOWN = wx.adv.DP_DROPDOWN
    else:
        WX_FONTSTYLE_NORMAL = wx.FONTWEIGHT_NORMAL
        WX_DP_DROPDOWN = wx.DP_DROPDOWN


class Classes:
    if isLatestVersion:
        DatePickerCtrl = wx.adv.DatePickerCtrl
    else:
        DatePickerCtrl = wx.DatePickerCtrl




def imageFromBitmap(bitmap):
    if isLatestVersion:
        return bitmap.ConvertToImage()
    else:
        return wx.ImageFromBitmap(bitmap)


def bitmapFromImage(image):
    if isLatestVersion:
        return wx.Bitmap(image)
    else:
        return wx.BitmapFromImage(image)







'''
Created on Jan 19, 2014
@author: Chris
'''

import sys

import wx

from gooey.gui import image_repository, events
from gooey.gui.lang.i18n import _
from gooey.gui.pubsub import pub
from gooey.gui.util import wx_util
from gooey.gui.windows import footer, header, layouts
from gooey.gui.windows.runtime_display_panel import RuntimeDisplay

YES = 5103
NO = 5104

class BaseWindow(wx.Frame):
  '''
  Primary Frame under which all sub-Panels are organized.
  '''

  def __init__(self, layout_type):
    wx.Frame.__init__(self, parent=None, id=-1)

    self.SetDoubleBuffered(True)

    # type of gui to render
    self.layout_type = layout_type

    # Components
    self.icon = None
    self.head_panel = None
    self.config_panel = None
    self.runtime_display = None
    self.foot_panel = None
    self.panels = None

    self._init_properties()
    self._init_components()
    self._do_layout()

    self.Bind(wx.EVT_SIZE, self.onResize)
    self.Bind(wx.EVT_CLOSE, self.onClose)

  @property
  def window_size(self):
    return self.GetSize()

  @window_size.setter
  def window_size(self, size_tuple):
    self.SetSize(size_tuple)

  @property
  def window_title(self):
    return self.GetTitle()

  @window_title.setter
  def window_title(self, title):
    self.SetTitle(title)

  @property
  def heading_title(self):
    return self.head_panel.title

  @heading_title.setter
  def heading_title(self, text):
    self.head_panel.title = text

  @property
  def heading_subtitle(self):
    return self.head_panel.subtitle

  @heading_subtitle.setter
  def heading_subtitle(self, text):
    self.head_panel.subtitle = text

  @property
  def required_section(self):
    return self.config_panel.main_content.required_section

  @property
  def optional_section(self):
    return self.config_panel.main_content.optional_section

  @property
  def progress_bar(self):
    return self.foot_panel.progress_bar

  def set_list_contents(self, contents):
    self.config_panel.sidebar.set_list_contents(contents)

  def set_display_font_style(self, style):
    # TODO: make this not stupid
    # TODO: _actual_ font support
    self.runtime_display.set_font_style(
      wx.MODERN if style == 'monospace' else wx.DEFAULT)

  def _init_properties(self):
    # self.SetTitle(self.build_spec['program_name'])
    # self.SetSize(self.build_spec['default_size'])
    # # self.SetMinSize((400, 300))
    self.icon = wx.Icon(image_repository.program_icon, wx.BITMAP_TYPE_ICO)
    self.SetIcon(self.icon)

  def _init_components(self):
    self.runtime_display = RuntimeDisplay(self)
    self.head_panel = header.FrameHeader(parent=self)
    self.foot_panel = footer.Footer(self)
    self.panels = [self.head_panel, self.config_panel, self.foot_panel]

  def _do_layout(self):
    sizer = wx.BoxSizer(wx.VERTICAL)
    sizer.Add(self.head_panel, 0, wx.EXPAND)
    sizer.Add(wx_util.horizontal_rule(self), 0, wx.EXPAND)

    if self.layout_type == layouts.COLUMN:
      self.config_panel = layouts.ColumnLayout(self)
    else:
      self.config_panel = layouts.FlatLayout(self)

    sizer.Add(self.config_panel, 1, wx.EXPAND)

    sizer.Add(self.runtime_display, 1, wx.EXPAND)

    self.runtime_display.Hide()
    sizer.Add(wx_util.horizontal_rule(self), 0, wx.EXPAND)
    sizer.Add(self.foot_panel, 0, wx.EXPAND)
    self.SetSizer(sizer)

    self.sizer = sizer

  def freeze(self):
    self.Freeze()

  def thaw(self):
    self.Thaw()

  def enable_stop_button(self):
    self.foot_panel.stop_button.Enable()

  def disable_stop_button(self):
    self.foot_panel.stop_button.Disable()

  def show(self, *args):
    '''
    Looks up the attribute across all available
    panels and calls `Show()`
    '''
    self._set_visibility('Show', *args)

  def hide(self, *args):
    '''
    Looks up the attribute across all available
    panels and calls `Show()`
    '''
    self._set_visibility('Hide', *args)

  def _set_visibility(self, action, *args):
    '''
    Checks for the existence `attr` on a given `panel` and
    performs `action` if found
    '''
    def _set_visibility(obj, attrs):
      for attr in attrs:
        if hasattr(obj, attr):
          instance = getattr(obj, attr)
          getattr(instance, action)()
          instance.Layout()
    for panel in [self, self.head_panel, self.foot_panel, self.config_panel]:
      _set_visibility(panel, args)

  def hide_all_buttons(self):
    self.foot_panel.hide_all_buttons()

  def update_console_async(self, msg):
    wx.CallAfter(self.runtime_display.append_text, msg)

  def update_progress_aync(self, progress, disable_animation=False):
    wx.CallAfter(self.UpdateProgressBar, progress, disable_animation)

  def onResize(self, evt):
    evt.Skip()

  def onClose(self, evt):
    if evt.CanVeto():
      evt.Veto()
    pub.send_message(str(events.WINDOW_CLOSE))

  def UpdateProgressBar(self, value, disable_animation=False):
    pb = self.foot_panel.progress_bar
    if value is None:
      return
    if value < 0:
      pb.Pulse()
    else:
      value = min(int(value), pb.GetRange())
      if pb.GetValue() != value:
        # Windows 7 progress bar animation hack
        # http://stackoverflow.com/questions/5332616/disabling-net-progressbar-animation-when-changing-value
        if disable_animation and sys.platform.startswith("win"):
          if pb.GetRange() == value:
            pb.SetValue(value)
            pb.SetValue(value-1)
          else:
            pb.SetValue(value+1)
        pb.SetValue(value)

  def show_dialog(self, title, content, style):
    dlg = wx.MessageDialog(None, content, title, style)
    result = dlg.ShowModal()
    dlg.Destroy()
    return result

  def show_missing_args_dialog(self):
    self.show_dialog(_('error_title'), _('error_required_fields'), wx.ICON_ERROR)

  def confirm_exit_dialog(self):
    result = self.show_dialog(_('sure_you_want_to_exit'), _('close_program'), wx.YES_NO)
    return result == YES

  def confirm_stop_dialog(self):
    result = self.show_dialog(_('sure_you_want_to_stop'), _('stop_task'), wx.YES_NO)
    return result == YES

if __name__ == '__main__':
  pass



'''
Created on Feb 10, 2014

@author: Chris
'''


class ParserError(Exception):
  """Thrown when the parser can't find argparse functions the client code"""
  pass


class ArgumentError(Exception):
  """Thrown when the parser is supplied with an incorrect argument format"""
  pass


if __name__ == '__main__':
  pass


'''
Created on Dec 23, 2013

@author: Chris
'''

import wx

from gooey.gui import imageutil, image_repository
from gooey.gui.util import wx_util


PAD_SIZE = 10


class FrameHeader(wx.Panel):
  def __init__(self, parent=None, **kwargs):
    wx.Panel.__init__(self, parent, **kwargs)
    self.SetDoubleBuffered(True)

    self._header = None
    self._subheader = None
    self.settings_img = None
    self.running_img = None
    self.check_mark = None
    self.error_symbol = None

    self._init_properties()
    self._init_components()
    self._do_layout()

  @property
  def title(self):
    return self._header.GetLabel()

  @title.setter
  def title(self, text):
    self._header.SetLabel(text)

  @property
  def subtitle(self):
    return self._subheader.GetLabel()

  @subtitle.setter
  def subtitle(self, text):
    self._subheader.SetLabel(text)

  def _init_properties(self):
    self.SetBackgroundColour('#ffffff')
    self.SetSize((30, 90))
    self.SetMinSize((120, 80))

  def _init_components(self):
    self._header = wx_util.h1(self, '')
    self._subheader = wx.StaticText(self, label='')

    self.settings_img = self._load_image(image_repository.config_icon, height=79)
    self.running_img = self._load_image(image_repository.running_icon, 79)
    self.check_mark = self._load_image(image_repository.success_icon, height=75)
    self.error_symbol = self._load_image(image_repository.error_icon, height=75)


  def _do_layout(self):
    vsizer = wx.BoxSizer(wx.VERTICAL)
    sizer = wx.BoxSizer(wx.HORIZONTAL)
    headings_sizer = self.build_heading_sizer()
    sizer.Add(headings_sizer, 1, wx.ALIGN_LEFT | wx.ALIGN_CENTER_HORIZONTAL | wx.EXPAND | wx.LEFT, PAD_SIZE)
    sizer.Add(self.settings_img, 0, wx.ALIGN_RIGHT | wx.EXPAND | wx.RIGHT, PAD_SIZE)
    sizer.Add(self.running_img, 0, wx.ALIGN_RIGHT | wx.EXPAND | wx.RIGHT, PAD_SIZE)
    sizer.Add(self.check_mark, 0, wx.ALIGN_RIGHT | wx.EXPAND | wx.RIGHT, PAD_SIZE)
    sizer.Add(self.error_symbol, 0, wx.ALIGN_RIGHT | wx.EXPAND | wx.RIGHT, PAD_SIZE)
    self.running_img.Hide()
    self.check_mark.Hide()
    self.error_symbol.Hide()
    vsizer.Add(sizer, 1, wx.EXPAND)
    self.SetSizer(vsizer)

  def _load_image(self, img_path, height=70):
    return imageutil.resize_bitmap(self, imageutil._load_image(img_path), height)

  def build_heading_sizer(self):
    sizer = wx.BoxSizer(wx.VERTICAL)
    sizer.AddStretchSpacer(1)
    sizer.Add(self._header, 0)
    sizer.Add(self._subheader, 0)
    sizer.AddStretchSpacer(1)
    return sizer




from gooey.gui.model import MyModel
from gooey.gui.presenter import Presenter
from gooey.gui.windows.base_window import BaseWindow


class Controller(object):

  def __init__(self, build_spec):
    self.model = MyModel(build_spec)
    self.view = BaseWindow(layout_type=self.model.layout_type)
    self.presentation = Presenter(self.view, self.model)
    self.presentation.initialize_view()


  def run(self):
    self.view.Show(True)




'''
Created on Dec 23, 2013

@author: Chris
'''

import wx
from gooey.gui.lang import i18n

from gooey.gui.three_to_four import Constants


class RuntimeDisplay(wx.Panel):
  '''
  Textbox displayed during the client program's execution.
  '''
  def __init__(self, parent, **kwargs):
    wx.Panel.__init__(self, parent, **kwargs)
    self._init_properties()
    self._init_components()
    self._do_layout()


  def set_font_style(self, style):
    pointsize = self.cmd_textbox.GetFont().GetPointSize()
    font = wx.Font(pointsize, style,
                   Constants.WX_FONTSTYLE_NORMAL, wx.FONTWEIGHT_BOLD, False)
    self.cmd_textbox.SetFont(font)

  def _init_properties(self):
    self.SetBackgroundColour('#F0F0F0')

  def _init_components(self):
    self.text = wx.StaticText(self, label=i18n._("status"))

    self.cmd_textbox = wx.TextCtrl(
      self, -1, "",
      style=wx.TE_MULTILINE | wx.TE_READONLY | wx.TE_RICH)


  def _do_layout(self):
    sizer = wx.BoxSizer(wx.VERTICAL)
    sizer.AddSpacer(10)
    sizer.Add(self.text, 0, wx.LEFT, 30)
    sizer.AddSpacer(10)
    sizer.Add(self.cmd_textbox, 1, wx.LEFT | wx.RIGHT | wx.BOTTOM | wx.EXPAND, 30)
    sizer.AddSpacer(20)
    self.SetSizer(sizer)

  def append_text(self, txt):
    self.cmd_textbox.AppendText(txt)




from argparse import ArgumentParser, _SubParsersAction
from argparse import _MutuallyExclusiveGroup, _ArgumentGroup

from gooey.gui.lang.i18n import _


class GooeySubParser(_SubParsersAction):
    def __init__(self, *args, **kwargs):
        super(GooeySubParser, self).__init__(*args, **kwargs)

# TODO: dedupe code
class GooeyArgumentGroup(_ArgumentGroup):
    def __init__(self, parser, widgets, *args, **kwargs):
        self.parser = parser
        self.widgets = widgets
        super(GooeyArgumentGroup, self).__init__(self.parser, *args, **kwargs)

    def add_argument(self, *args, **kwargs):
        widget = kwargs.pop('widget', None)
        metavar = kwargs.pop('metavar', None)
        super(GooeyArgumentGroup, self).add_argument(*args, **kwargs)
        self.parser._actions[-1].metavar = metavar
        self.widgets[self.parser._actions[-1].dest] = widget


class GooeyMutuallyExclusiveGroup(_MutuallyExclusiveGroup):
    def __init__(self, parser, widgets, *args, **kwargs):
        self.parser = parser
        self.widgets = widgets
        super(GooeyMutuallyExclusiveGroup, self).__init__(self.parser, *args, **kwargs)

    def add_argument(self, *args, **kwargs):
        widget = kwargs.pop('widget', None)
        metavar = kwargs.pop('metavar', None)
        super(GooeyMutuallyExclusiveGroup, self).add_argument(*args, **kwargs)
        self.parser._actions[-1].metavar = metavar
        self.widgets[self.parser._actions[-1].dest] = widget


class GooeyParser(object):
    def __init__(self, **kwargs):
        self.__dict__['parser'] = ArgumentParser(**kwargs)
        self.widgets = {}

    @property
    def _mutually_exclusive_groups(self):
        return self.parser._mutually_exclusive_groups

    @property
    def _actions(self):
        return self.parser._actions

    @property
    def description(self):
        return self.parser.description

    def add_argument(self, *args, **kwargs):
        widget = kwargs.pop('widget', None)
        metavar = kwargs.pop('metavar', None)

        if widget and widget == 'Listbox':
            if not 'nargs' in kwargs or kwargs['nargs'] not in ['*', '+']:
                raise ValueError(
                    'Gooey\'s Listbox widget requires that nargs be specified.\n'
                    'Nargs must be set to either `*` or `+` (e.g. nargs="*")'
                )
        self.parser.add_argument(*args, **kwargs)
        self.parser._actions[-1].metavar = metavar
        self.widgets[self.parser._actions[-1].dest] = widget

    def add_mutually_exclusive_group(self, **kwargs):
        group = GooeyMutuallyExclusiveGroup(self.parser, self.widgets, **kwargs)
        self.parser._mutually_exclusive_groups.append(group)
        return group

    def add_argument_group(self, *args, **kwargs):
        group = GooeyArgumentGroup(self.parser, self.widgets, **kwargs)
        self.parser.add_argument_group(*args, **kwargs)
        return group

    def parse_args(self, args=None, namespace=None):
        return self.parser.parse_args(args, namespace)

    def add_subparsers(self, **kwargs):
        if self._subparsers is not None:
            self.error(_('cannot have multiple subparser arguments'))

        # add the parser class to the arguments if it's not present
        kwargs.setdefault('parser_class', type(self))

        if 'title' in kwargs or 'description' in kwargs:
            title = _(kwargs.pop('title', 'subcommands'))
            description = _(kwargs.pop('description', None))
            self._subparsers = self.add_argument_group(title, description)
        else:
            self._subparsers = self._positionals

        # prog defaults to the usage message of this parser, skipping
        # optional arguments and with no "usage:" prefix
        if kwargs.get('prog') is None:
            formatter = self._get_formatter()
            positionals = self._get_positional_actions()
            groups = self._mutually_exclusive_groups
            formatter.add_usage(self.usage, positionals, groups, '')
            kwargs['prog'] = formatter.format_help().strip()

        # create the parsers action and add it to the positionals list
        parsers_class = self._pop_action_class(kwargs, 'parsers')
        action = parsers_class(option_strings=[], **kwargs)
        self._subparsers._add_action(action)

        # return the created parsers action
        return action

    def __getattr__(self, item):
        return getattr(self.parser, item)

    def __setattr__(self, key, value):
        return setattr(self.parser, key, value)



'''

Simple utlity to generate the Json English language file

Created on Jan 25, 2014

@author: Chris
'''

import json


if __name__ == '__main__':
  english = {  # Header Messages
               'settings_title': 'Settings',
               'running_title': 'Running',
               'running_msg': 'Please wait while the application performs its tasks. ' +
                              '\nThis may take a few moments',
               'finished_title': 'Finished',
               'finished_msg': 'All done! You may now safely close the program.',  # Footer buttons
               'cancel': 'Cancel',
               'close': 'Close',
               'start': 'Start',  # simple config panel
               'simple_config': 'Enter Command Line Arguments',  # Advanced config panel
               'required_args_msg': 'Required Arguments',
               'optional_args_msg': 'Optional Arguments',  # popup dialogs
               "sure_you_want_to_exit": "Are you sure you want to exit?",
               'close_program': 'Close program?',
               'sure_you_want_to_stop': 'Are you sure you want to stop the task? ' +
                                        '\nInterruption can corrupt your data!',
               'stop_task': 'Stop task?',
               'status': 'Status',
               'uh_oh': '''
Uh oh! Looks like there was a problem.
Copy the text from status window to let your developer know what went wrong.
''',
               'error_title': "Error",
               'execution_finished': 'Execution finished',
               'success_message': 'Program completed sucessfully!',

  }

  with open('english.json', 'wb') as f:
    f.write(json.dumps(english, indent=4, sort_keys=True))


'''
Created on Jan 20, 2014

@author: Chris
'''

import wx
from gooey.gui.three_to_four import imageFromBitmap, bitmapFromImage




def _load_image(image_path):
    try:
        return wx.Bitmap(image_path)
    except:
        raise IOError('Invalid Image path')


def resize_bitmap(parent, _bitmap, target_height):
    '''
    Resizes a bitmap to a height of 89 pixels (the
    size of the top panel), while keeping aspect ratio
    in tact
    '''
    image = imageFromBitmap(_bitmap)
    _width, _height = image.GetSize()
    if _height < target_height:
        return wx.StaticBitmap(parent, -1, bitmapFromImage(image))
    ratio = float(_width) / _height
    image = image.Scale(target_height * ratio, target_height, wx.IMAGE_QUALITY_HIGH)
    return wx.StaticBitmap(parent, -1, bitmapFromImage(image))


if __name__ == '__main__':
    pass
